// beans/Timer.java

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.util.Vector;
import java.util.Enumeration;

public class Timer
    implements java.io.Serializable
{
    public static final String PROP_DELAY = "delay";
    public static final long DEFAULT_DELAY = 1000;
    
    transient private TimerThread timerThread;
    transient private Vector<TimerListener> listeners;

    private PropertyChangeSupport propertySupport;
    private boolean running;
    private long delay;

    // needs to be bumped if there are incompabible changes
    private static final long serialVersionUID = 1L;

    public Timer()
    {
        delay = DEFAULT_DELAY;
        propertySupport = new PropertyChangeSupport(this);
        start();
    }

    public synchronized void start()
    {
        if (running) return;
        timerThread = new TimerThread();
        running = true;
        timerThread.start();
    }

    public synchronized void stop()     // deprecated
    {
        if (! running) return;
        timerThread.stop();
        timerThread = null;
        running = false;
    }

    public long getDelay()
    {
        return delay;
    }

    public void setDelay(long value)
    {
        if (delay == value) return;
        long oldValue = delay;
        delay = value;
        propertySupport.firePropertyChange(PROP_DELAY,
                                           new Long(oldValue),
                                           new Long(delay));
    }

    public void addPropertyChangeListener(PropertyChangeListener pcl)
    {
        propertySupport.addPropertyChangeListener(pcl);
    }

    public void removePropertyChangeListener(PropertyChangeListener pcl)
    {
        propertySupport.removePropertyChangeListener(pcl);
    }

    public void addTimerListener(TimerListener l)
    {
        if (listeners == null) listeners = new Vector<TimerListener>();
        listeners.addElement(l);
    }

    public void removeTimerListener(TimerListener l)
    {
        if (listeners == null) return;
        listeners.removeElement(l);
    }

    private void fireTimerEvent()
    {
	Vector l;

	if (listeners == null) return;
	
        synchronized(this) {
            l = (Vector) listeners.clone();
        }

        for (Enumeration e = l.elements(); e.hasMoreElements();) {
            TimerListener tl = (TimerListener) e.nextElement();
            tl.onTime(new ActionEvent(this, 
                                      ActionEvent.ACTION_PERFORMED,
                                      "onTime"));
        }
    }

    class TimerThread extends Thread
    {
        public void run() {
            while (true) {
                try {
                    sleep(delay);
                } catch (InterruptedException e) {}
                fireTimerEvent();
            }
        }
    }
}
